import { Directive, ElementRef, Injectable, Input, NgModule, Renderer2, defineInjectable, inject } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HTTP_INTERCEPTORS, HttpClientModule, HttpErrorResponse, HttpResponse } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { catchError, share, tap } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
var CounterService = /** @class */ (function () {
    function CounterService() {
        var _this = this;
        this._connectionCounter = 0;
        this._observable = Observable.create(function (subscriber) {
            _this._subscriber = subscriber;
        }).pipe(share());
    }
    Object.defineProperty(CounterService.prototype, "observable", {
        get: /**
         * @return {?}
         */
        function () {
            return this._observable;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    CounterService.prototype.increment = /**
     * @return {?}
     */
    function () {
        this._connectionCounter += 1;
        this._next();
    };
    /**
     * @return {?}
     */
    CounterService.prototype.decrement = /**
     * @return {?}
     */
    function () {
        this._connectionCounter -= 1;
        this._next();
    };
    /**
     * @return {?}
     */
    CounterService.prototype._next = /**
     * @return {?}
     */
    function () {
        if (this._subscriber) {
            this._subscriber.next(this._connectionCounter);
        }
    };
    CounterService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] },
    ];
    /** @nocollapse */
    CounterService.ctorParameters = function () { return []; };
    /** @nocollapse */ CounterService.ngInjectableDef = defineInjectable({ factory: function CounterService_Factory() { return new CounterService(); }, token: CounterService, providedIn: "root" });
    return CounterService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
var HttpClientBusyInterceptor = /** @class */ (function () {
    function HttpClientBusyInterceptor(_counter) {
        this._counter = _counter;
    }
    /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    HttpClientBusyInterceptor.prototype.intercept = /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    function (req, next) {
        var _this = this;
        this._counter.increment();
        return next.handle(req).pipe(tap(function (evt) {
            if (evt instanceof HttpResponse) {
                _this._counter.decrement();
            }
        }), catchError(function (err) {
            if (err instanceof HttpErrorResponse) {
                _this._counter.decrement();
            }
            return throwError(err);
        }));
    };
    HttpClientBusyInterceptor.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] },
    ];
    /** @nocollapse */
    HttpClientBusyInterceptor.ctorParameters = function () { return [
        { type: CounterService }
    ]; };
    /** @nocollapse */ HttpClientBusyInterceptor.ngInjectableDef = defineInjectable({ factory: function HttpClientBusyInterceptor_Factory() { return new HttpClientBusyInterceptor(inject(CounterService)); }, token: HttpClientBusyInterceptor, providedIn: "root" });
    return HttpClientBusyInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
var HttpClientBusyDirective = /** @class */ (function () {
    function HttpClientBusyDirective(_el, _renderer, _counter) {
        this._el = _el;
        this._renderer = _renderer;
        this._counter = _counter;
        this._httpClientBusyConfig = {};
    }
    /**
     * @param {?} count
     * @return {?}
     */
    HttpClientBusyDirective.prototype._updateStatus = /**
     * @param {?} count
     * @return {?}
     */
    function (count) {
        /** @type {?} */
        var busy = count > 0;
        /** @type {?} */
        var hasIdle = this._el.nativeElement.classList.contains(this._httpClientBusyConfig.idle);
        /** @type {?} */
        var hasBusy = this._el.nativeElement.classList.contains(this._httpClientBusyConfig.busy);
        if (busy) {
            if (this._httpClientBusyConfig.idle && hasIdle) {
                this._renderer.removeClass(this._el.nativeElement, this._httpClientBusyConfig.idle);
            }
            if (this._httpClientBusyConfig.busy && !hasBusy) {
                this._renderer.addClass(this._el.nativeElement, this._httpClientBusyConfig.busy);
            }
        }
        else {
            if (this._httpClientBusyConfig.busy && hasBusy) {
                this._renderer.removeClass(this._el.nativeElement, this._httpClientBusyConfig.busy);
            }
            if (this._httpClientBusyConfig.idle && !hasIdle) {
                this._renderer.addClass(this._el.nativeElement, this._httpClientBusyConfig.idle);
            }
        }
    };
    /**
     * @return {?}
     */
    HttpClientBusyDirective.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._httpClientBusyConfig.idle = this.httpClientBusy.idle || 'fade';
        this._httpClientBusyConfig.busy = this.httpClientBusy.busy || '';
        this._updateStatus(0);
        this._autoUnsubscribe = this._counter.observable.subscribe(function (next) { return _this._updateStatus(next); });
    };
    /**
     * @return {?}
     */
    HttpClientBusyDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this._autoUnsubscribe.unsubscribe();
    };
    HttpClientBusyDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[httpClientBusy]'
                },] },
    ];
    /** @nocollapse */
    HttpClientBusyDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: CounterService }
    ]; };
    HttpClientBusyDirective.propDecorators = {
        httpClientBusy: [{ type: Input }]
    };
    return HttpClientBusyDirective;
}());
/**
 * @record
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
var HttpClientBusyModule = /** @class */ (function () {
    function HttpClientBusyModule() {
    }
    /**
     * @return {?}
     */
    HttpClientBusyModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: HttpClientBusyModule
        };
    };
    HttpClientBusyModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, HttpClientModule],
                    declarations: [HttpClientBusyDirective],
                    exports: [HttpClientBusyDirective],
                    providers: [{ provide: HTTP_INTERCEPTORS, useClass: HttpClientBusyInterceptor, multi: true }]
                },] },
    ];
    return HttpClientBusyModule;
}());

export { HttpClientBusyModule, CounterService, HttpClientBusyDirective, HttpClientBusyInterceptor };
